module Functions.Pandoc
  ( readMarkdown
  , writeHtml
  , extractTitle
  ) where

import Types (IsFunction(..), Token)
import DependencyGenerator (DepGenM, TokenableTo(..), runFunction)

import Data.Text (Text)
import Text.Pandoc.Definition (Pandoc)
import qualified Text.Pandoc.Definition as PD
import qualified Text.Pandoc.Shared as PS
import qualified Text.Pandoc as P

runPandoc :: P.PandocPure a -> a
runPandoc m = case P.runPure m of
  Left e -> error ("unexpected pandoc error: " ++ show e)
  Right result -> result

data ReadMarkdown = ReadMarkdown deriving Show
instance IsFunction ReadMarkdown Text Pandoc where
  evalFunction ReadMarkdown contents =
    let settings = P.def { P.readerExtensions = P.extensionsFromList [ P.Ext_raw_html ] }
    in runPandoc $ P.readMarkdown settings contents

readMarkdown :: TokenableTo Text a => a -> DepGenM (Token Pandoc)
readMarkdown a = runFunction ReadMarkdown =<< toToken a


data WriteHtml = WriteHtml deriving Show
instance IsFunction WriteHtml Pandoc Text where
  evalFunction WriteHtml pandoc =
    runPandoc
    $ P.writeHtml5String P.def pandoc

writeHtml :: TokenableTo Pandoc a => a -> DepGenM (Token Text)
writeHtml a = runFunction WriteHtml =<< toToken a


data ExtractTitle = ExtractTitle deriving Show
instance IsFunction ExtractTitle Pandoc Text where
  evalFunction ExtractTitle (PD.Pandoc _ blocks) = case blocks of
    (PD.Header 1 _ inlines : _) -> PS.stringify inlines
    _ -> error "unexpected"

extractTitle :: TokenableTo Pandoc a => a -> DepGenM (Token Text)
extractTitle a = runFunction ExtractTitle =<< toToken a
